@@ -1,4 +1,5 @@
 import argparse
+import inspect
 import os
 import re
 import sys
@@ -22,7 +23,8 @@ from ptera.selfless import PreState, PteraNameError
 
 from .config import ConfigFile
 
-Argument = tag.Argument
+Option = tag.Option
+Argument = Option
 
 
 _count = count()
@@ -194,6 +196,7 @@ class Configurator:
             positional=False,
             metavar=None,
             negate=None,
+            negate_doc=None,
             group=None,
             name=name,
             optname=optname,
@@ -216,11 +219,23 @@ class Configurator:
                     if command in ["alias", "aliases"]:
                         opts.aliases.extend(re.split(r"[ ,;]+", arg))
                     elif command in ["option", "options"]:
+                        arg = arg or ""
                         opts.aliases = re.split(r"[ ,;]+", arg)
                     elif command == "group":
                         opts.group = arg
                     elif command == "negate":
-                        opts.negate = arg or True
+                        if arg:
+                            opts.negate = re.split(r"[ ,;]+", arg)
+                        else:
+                            opts.negate = [f"--no-{optname}"]
+                        opts.aliases = []
+                    elif command == "false-options":
+                        if arg:
+                            opts.negate = re.split(r"[ ,;]+", arg)
+                        else:
+                            opts.negate = [f"--no-{optname}"]
+                    elif command == "false-options-doc":
+                        opts.negate_doc = arg
                     elif command == "metavar":
                         opts.metavar = arg
                     elif command == "action":
@@ -239,7 +254,9 @@ class Configurator:
                     new_entry.append(line)
             opts.doc.append("\n".join(new_entry))
 
-        opts.has_default_opt_name = (opts.aliases[0] == default_opt),
+        opts.has_default_opt_name = (
+            (opts.aliases and opts.aliases[0] == default_opt),
+        )
 
         for x in data.values():
             x["coleo_options"] = opts
@@ -289,7 +306,6 @@ class Configurator:
             aliases = entry.aliases
             nargs = entry.nargs
             optdoc = entry.doc
-            optname = entry.optname
 
             if entry.group:
                 if entry.group not in groups:
@@ -304,21 +320,26 @@ class Configurator:
                 if group is self.argparser and entry.negate is not None:
                     group = self.argparser.add_mutually_exclusive_group()
 
-                group.add_argument(
-                    *aliases,
-                    dest=name,
-                    action="store_true",
-                    help="; ".join(optdoc),
-                )
-
-                if entry.negate is not None:
+                if aliases:
                     group.add_argument(
-                        f"--no-{optname}",
+                        *aliases,
+                        dest=name,
+                        action="store_true",
+                        help="; ".join(optdoc),
+                    )
+
+                if entry.negate:
+                    if entry.negate_doc:
+                        doc = entry.negate_doc
+                    elif aliases:
+                        doc = f"Set {aliases[0]} to False"
+                    else:
+                        doc = "; ".join(optdoc)
+                    group.add_argument(
+                        *entry.negate,
                         dest=name,
                         action="store_false",
-                        help=f"Set --{optname} to False"
-                        if entry.negate is True
-                        else entry.negate,
+                        help=doc,
                     )
             else:
                 if entry.positional:
@@ -383,7 +404,7 @@ def _getdoc(obj):
         return getattr(obj, "__doc__", None)
 
 
-def _auto_cli_helper(parser, entry, **kwargs):
+def _make_cli_helper(parser, entry, **kwargs):
     if isinstance(entry, dict):
         subparsers = parser.add_subparsers()
         for name, subentry in entry.items():
@@ -392,7 +413,19 @@ def _auto_cli_helper(parser, entry, **kwargs):
             subparser = subparsers.add_parser(
                 name, help=_getdoc(subentry), argument_default=argparse.SUPPRESS
             )
-            _auto_cli_helper(subparser, subentry)
+            _make_cli_helper(subparser, subentry)
+
+    elif inspect.isclass(entry):
+        structure = {"__doc__": entry.__doc__}
+        for name, entry2 in vars(entry).items():
+            if isinstance(entry2, FunctionType):
+                entry2 = tooled(entry2)
+                setattr(entry, name, entry2)
+                structure[name] = entry2
+            elif isinstance(entry2, (PteraFunction, dict, type)):
+                structure[name] = entry2
+        return _make_cli_helper(parser, structure, **kwargs)
+
     else:
         if isinstance(entry, FunctionType):
             entry = tooled(entry)
@@ -418,7 +451,7 @@ def setvars(**values):
     return _setvars(values, tag=Argument)
 
 
-def auto_cli(
+def make_cli(
     entry,
     args=(),
     *,
@@ -428,7 +461,6 @@ def auto_cli(
     description=None,
     eval_env=None,
     expand=None,
-    print_result=True,
     return_split=False,
 ):
     if expand is None or isinstance(expand, str):
@@ -438,7 +470,7 @@ def auto_cli(
         description=description or _getdoc(entry),
         argument_default=argparse.SUPPRESS,
     )
-    _auto_cli_helper(parser, entry, tag=tag, eval_env=eval_env)
+    _make_cli_helper(parser, entry, tag=tag, eval_env=eval_env)
     opts = parse_options(parser, argv=argv, expand=expand)
     cfg, fn = getattr(opts, "#cfg", (None, None))
     if cfg is None:
@@ -460,11 +492,20 @@ def auto_cli(
                     sys.exit(1)
                 else:
                     raise
-            if print_result and result is not None:
-                print(result)
             return result
 
-    if return_split:
-        return opts, thunk
-    else:
-        return thunk(opts)
+    return opts, thunk
+
+
+def run_cli(*args, **kwargs):
+    opts, thunk = make_cli(*args, **kwargs)
+    return thunk(opts)
+
+
+def auto_cli(fn, *args, **kwargs):  # pragma: no cover
+    if inspect.isfunction(fn):
+        fn = tooled(fn)
+    result = run_cli(fn, *args, **kwargs)
+    if result is not None:
+        print(result)
+    return fn
